@@ -1,3 +1,5 @@
+#ifndef INCLUDED_stripgfx_h
+#define INCLUDED_stripgfx_h
 
 #define NO_GRAPHICS 1
 #define DEC_GRAPHICS 2
@@ -5,3 +7,5 @@
 
 void populate_gfx_array (int gfxset);
 unsigned char strip_gfx (unsigned char inchar);
+
+#endif /* !INCLUDED_stripgfx_h */
